/**
 * Copyright (C) 2010 <dueni(at)gmx.net>
 * 
 * Licensed under the Apache License, Version 2.0 (the "License"); you may not use this file except
 * in compliance with the License. You may obtain a copy of the License at
 * 
 * http://www.apache.org/licenses/LICENSE-2.0
 * 
 * Unless required by applicable law or agreed to in writing, software distributed under the License
 * is distributed on an "AS IS" BASIS, WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express
 * or implied. See the License for the specific language governing permissions and limitations under
 * the License.
 */

package net.java.dev.faces.chain;

import javax.faces.application.Application;
import javax.faces.event.AbortProcessingException;
import javax.faces.event.PostConstructApplicationEvent;
import javax.faces.event.PreDestroyApplicationEvent;
import javax.faces.event.SystemEvent;
import javax.faces.event.SystemEventListener;
import javax.servlet.ServletContextEvent;
import javax.servlet.ServletContextListener;

import org.apache.commons.chain.CatalogFactory;

/**
 * <code>FacesChainInitializer</code> is used to initialize faces-chain by loading all
 * "*commons-chain*.xml" config files into commons-chain's CatalogFactory.
 * 
 * <p>
 * For JSF 2.0 this class acts as SystemEventListener to listen for the
 * {@link PostConstructApplicationEvent} and {@link PreDestroyApplicationEvent} to load or reset
 * commons-chain configuration.
 * </p>
 * 
 * <p>
 * For usage with JSF versions before 2.0 this class also implements ServletContextListener so it
 * may be configured as a context-listener in web.xml to load or reset commons-chain configuration.
 * </p>
 * 
 * <p>
 * Loading of multiple chain-config.xml files like "myapp-chain-config.xml" or
 * "chain-config-abc.xml" is supported for locations where the base names
 * "META-INF/chain-config.xml" or "WEB-INF/chain-config.xml" are found on the class-path. If the
 * base name "chain-config.xml" is missing, the initializer will not look for alike names and the
 * configuration files will not be loaded. All chain-config.xml files must be located in either
 * "META-INF" or "WEB-INF" directory of any jar on the class-path or the web application.
 * </p>
 * 
 */
public class FacesChainInitializer implements ServletContextListener, SystemEventListener {

	/**
	 * Reset commons-chain configuration on web application shutdown.
	 * 
	 * @see ServletContextListener#contextDestroyed(ServletContextEvent)
	 */
	public void contextDestroyed(ServletContextEvent servletContextEvent) {
		resetCommonsChain();
	}

	/**
	 * Load commons-chain configuration files on web application initialization.
	 * 
	 * @see ServletContextListener#contextInitialized(ServletContextEvent)
	 */
	public void contextInitialized(ServletContextEvent servletcontextevent) {
		initCommonsChain();
	}

	/**
	 * Return true if the omitting object is of type {@link Application}, false otherwise.
	 * 
	 * @return true if the omitting object is of type {@link Application}, false otherwise.
	 */
	public boolean isListenerForSource(Object source) {
		return javax.faces.application.Application.class.isAssignableFrom(source.getClass());
	}

	/**
	 * Process {@link PostConstructApplicationEvent} to load commons-chain configuration files on web
	 * application initialization or {@link PreDestroyApplicationEvent} to reset commons-chain
	 * configuration on application shutdown.
	 */
	public void processEvent(SystemEvent event) throws AbortProcessingException {
		if (event instanceof PostConstructApplicationEvent) {
			initCommonsChain();
		} else if (event instanceof PreDestroyApplicationEvent) {
			resetCommonsChain();
		}
	}

	/**
	 * Performs commons-chain initialization by loading all "*chain-config*.xml" files into
	 * {@link CatalogFactory}.
	 * 
	 */
	private void initCommonsChain() {
		FacesChainUtil.loadCommandChainConfigs();

		CatalogFactory factory = CatalogFactory.getInstance();
		if (!factory.getNames().hasNext() && factory.getCatalog() == null) {
			FacesChainUtil.LOG.warning("Could not load any commons-chain Catalog definition!");
		}
	}

	/**
	 * Performs reset of commons-chain configuration.
	 * 
	 */
	private void resetCommonsChain() {
		CatalogFactory.clear();
	}

}
