/**
 * Copyright (C) 2010 <dueni(at)gmx.net>
 * 
 * Licensed under the Apache License, Version 2.0 (the "License"); you may not use this file except
 * in compliance with the License. You may obtain a copy of the License at
 * 
 * http://www.apache.org/licenses/LICENSE-2.0
 * 
 * Unless required by applicable law or agreed to in writing, software distributed under the License
 * is distributed on an "AS IS" BASIS, WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express
 * or implied. See the License for the specific language governing permissions and limitations under
 * the License.
 */

package net.java.dev.faces.chain;

import java.lang.reflect.Constructor;
import java.lang.reflect.Method;
import java.net.URL;
import java.util.Arrays;
import java.util.Iterator;
import java.util.List;
import java.util.logging.Level;
import java.util.logging.Logger;

import javax.faces.context.ExternalContext;
import javax.faces.context.FacesContext;

import org.apache.commons.chain.Catalog;
import org.apache.commons.chain.CatalogFactory;
import org.apache.commons.chain.Command;
import org.apache.commons.chain.config.ConfigParser;
import org.apache.commons.chain.generic.LookupCommand;
import org.apache.commons.chain.impl.ChainBase;
import org.apache.commons.digester.RuleSet;

/**
 * <code>FacesChainUtil</code> provides several functions requiring access to ClassLoaders and
 * resources.
 * 
 * @author dueni
 * 
 */
public class FacesChainUtil {

	/**
	 * Private Logger instance.
	 */
	static final Logger LOG = Logger.getLogger(FacesChainUtil.class.getPackage().getName());

	/**
	 * web.xml context-param to list the loaded catalog.
	 */
	public static final String LOG_CATALOGS_CONTEXT_PARAM_KEY =
			"net.java.dev.faces.chain.logCatalogs";

	/**
	 * Load all *chain-config*.xml configuration files from META-INF/ and WEB-INF/ class path. This
	 * uses {@link ResourceFinder#findResources(String, String)} to find the resources and passes each
	 * config file to {@link ConfigParser}.
	 * 
	 */
	public static void loadCommandChainConfigs() {
		loadCommandChainConfigs(null);
	}

	/**
	 * Load all *chain-config*.xml configuration files from META-INF/ and WEB-INF/ class path. This
	 * uses {@link ResourceFinder#findResources(String, String)} to find the resources and passes each
	 * config file to {@link ConfigParser}.
	 * 
	 * @param lookupPaths
	 *          list of paths to look for chain-config.xml files.
	 * 
	 */
	public static void loadCommandChainConfigs(List<String> lookupPaths) {
		ConfigParser parser = new ConfigParser();
		String className = FacesChainFactory.getInstance().getConfigRuleSetClassName();
		parser.setRuleSet((RuleSet)newInstance(className));
		List<URL> chainConfigs =
				ResourceFinder.getInstance().findResources("chain-config", "xml", lookupPaths);
		for (URL chainConfig : chainConfigs) {
			try {
				parser.parse(chainConfig);
			} catch (Throwable t) {
				// TODO LOG/throw exception
				t.printStackTrace();
			}
		}

		logCatalogs();
	}

	/**
	 * Log Catalog setup with {@link Level#CONFIG} if in the context-param
	 * {@link #LOG_CATALOGS_CONTEXT_PARAM_KEY} is set to true in web.xml.
	 */
	public static void logCatalogs() {
		ExternalContext ec = FacesContext.getCurrentInstance().getExternalContext();
		if (Boolean.valueOf(ec.getInitParameter(LOG_CATALOGS_CONTEXT_PARAM_KEY))) {
			CatalogFactory cf = CatalogFactory.getInstance();
			Catalog cat = cf.getCatalog();
			logCatalog("(default)", cat);
			Iterator<?> names = cf.getNames();
			while (names.hasNext()) {
				String name = (String)names.next();
				logCatalog(name, cf.getCatalog(name));
			}
		}
	}

	/**
	 * Log the loaded catalogs if logging is enabled.
	 * 
	 * @param catalogName
	 *          name of the catalog to log.
	 * @param cat
	 *          the Catalog to log it's chain setup.
	 */
	private static void logCatalog(String catalogName, Catalog cat) {
		if (cat != null) {
			StringBuilder sb = new StringBuilder(1024);
			sb.append(" \n");
			sb.append("=== list chain configuration for catalog; '").append(catalogName).append(
					"' === begin ===\n");
			Iterator<?> names = cat.getNames();
			Command c = null;
			String name = "";
			while (names.hasNext()) {
				name = (String)names.next();
				c = cat.getCommand(name);
				sb.append("Command: '").append(name).append("' type: '").append(c.getClass().getName())
						.append("'\n");
				logChain(1, sb, c);
			}

			sb.append("=== list chain configuration for catalog: '").append(catalogName).append(
					"' === end ===");
			LOG.config(sb.toString());
		}
	}

	/**
	 * Add chain entries to StringBuilder for the log.
	 * 
	 * @param indent
	 *          indentation level.
	 * @param sb
	 *          resulting StringBuilder.
	 * @param command
	 *          the command to log.
	 */
	private static void logChain(int indent, StringBuilder sb, Command command) {
		if (command instanceof ChainBase) {
			Class<?> clazz = command.getClass();
			try {
				while (clazz != ChainBase.class) {
					clazz = clazz.getSuperclass();
				}
				Method getCommands = clazz.getDeclaredMethod("getCommands", new Class[] {});
				getCommands.setAccessible(true);
				Command[] commands = (Command[])getCommands.invoke(command, new Object[] {});
				for (Command c : commands) {
					prefix(indent, sb);
					sb.append("Command: '(none)' type: '").append(c.getClass().getName()).append("'\n");
					logChain(indent + 1, sb, c);
				}
			} catch (Exception e) {
				// Stacktrace not needed
				e.printStackTrace();
			}
		} else if (LookupCommand.class.isAssignableFrom(command.getClass())) {
			LookupCommand lc = (LookupCommand)command;
			String catalogName = lc.getCatalogName();
			String name = lc.getName();
			prefix(indent, sb);
			sb.append("Lookup for command: '").append(name).append("' in catalog: '").append(catalogName)
					.append("'.\n");
		}
	}

	/**
	 * Prefix a line according to level.
	 * 
	 * @param indent
	 *          indentation level.
	 * @param sb
	 *          StringBuilder for the log.
	 */
	private static void prefix(int indent, StringBuilder sb) {
		for (int i = indent; i > 0; i--) {
			if (i == 1) {
				sb.append("|- ");
			} else {
				sb.append("|  ");
			}
		}
	}

	/**
	 * Get ClassLoader from either current Thread or FacesChainUtil class.
	 * 
	 * @return ClassLoader from either current Thread or FacesChainUtil class.
	 */
	public static ClassLoader getClassLoader() {
		ClassLoader cl = Thread.currentThread().getContextClassLoader();
		if (cl == null) {
			cl = FacesChainUtil.class.getClassLoader();
		}
		return cl;
	}

	/**
	 * Return the Constructor for the named class accepting the given parameterTypes.
	 * 
	 * @param className
	 *          name of the class to load.
	 * @param parameterTypes
	 *          parameter types to lookup the wanted constructor.
	 * @return constructor for parameter types.
	 * @throws IllegalArgumentException
	 *           if class or wanted constructor cannot be found.
	 */
	public static Constructor<?> getConstructor(String className, Class<?>... parameterTypes) {
		try {
			Class<?> clazz = getClassLoader().loadClass(className);
			return clazz.getConstructor(parameterTypes);
		} catch (Exception e) {
			throw new IllegalArgumentException("Could not find constructor supporting arguments "
					+ Arrays.asList(parameterTypes) + " on class '" + className + "'!", e);
		}
	}

	/**
	 * Return new instance of the named class.
	 * 
	 * @param className
	 *          the name of the class to return an instance of.
	 * @param parameters
	 *          optional list of parameters - if null, default constructor will be used, otherwise
	 *          constructor for given parameters is looked up.
	 * @return new instance of the named class.
	 * @throws IllegalArgumentException
	 *           if the given class could not be loaded or no instance could be created.
	 */
	public static Object newInstance(String className, Object... parameters) {
		Object result = null;
		try {
			Class<?> clazz = FacesChainUtil.getClassLoader().loadClass(className);
			if (parameters == null || parameters.length == 0) {
				result = clazz.newInstance();
			} else {
				Class<?>[] types = new Class<?>[parameters.length];
				for (int i = 0; i < types.length; i++) {
					types[i] = parameters[i].getClass();
				}
				Constructor<?> ctor = clazz.getConstructor(types);
				result = ctor.newInstance(parameters);
			}
		} catch (Exception e) {
			throw new IllegalArgumentException("Could not instanciate '" + className + "'!", e);
		}
		return result;
	}

}
