/**
 * Copyright (C) 2010 <dueni(at)gmx.net>
 * 
 * Licensed under the Apache License, Version 2.0 (the "License"); you may not use this file except
 * in compliance with the License. You may obtain a copy of the License at
 * 
 * http://www.apache.org/licenses/LICENSE-2.0
 * 
 * Unless required by applicable law or agreed to in writing, software distributed under the License
 * is distributed on an "AS IS" BASIS, WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express
 * or implied. See the License for the specific language governing permissions and limitations under
 * the License.
 */

package net.java.dev.faces.chain.dynamic;

import java.util.ArrayList;
import java.util.List;

import net.java.dev.faces.chain.FacesChainFactory;
import net.java.dev.faces.chain.NamedCommand;

import org.apache.commons.chain.Command;
import org.apache.commons.chain.Context;
import org.apache.commons.chain.Filter;
import org.apache.commons.chain.impl.ChainBase;
import org.apache.commons.digester.RuleSet;

/**
 * <code>DynamicChain</code> is an test example of how to use a custom {@link RuleSet} to let
 * commons-chain use my custom chain implementation instead of the provided ChainBase. This allows
 * do e.g. manipulate the commands in the chain before the chain is processed.
 * 
 * @author dueni
 * 
 */
public class DynamicChain extends ChainBase implements NamedCommand {

	private String name;

	/**
	 * Return the name of the chain.
	 * 
	 * @return the name of the chain.
	 */
	public String getName() {
		return name;
	}

	/**
	 * Set the name of the chain.
	 * 
	 * @param name
	 *          the name of the chain.
	 */
	public void setName(String name) {
		this.name = name;
	}

	/**
	 * Override execute method of ChainBase to create a local copy of the commands list of the chain
	 * and allow to manipulate the local copy of the chain before it is processed.
	 * 
	 * @see Command#execute(Context).
	 */
	@Override
	public boolean execute(Context context) throws Exception {
		// Verify our parameters
		if (context == null) {
			throw new IllegalArgumentException();
		}

		List<Command> commandList = getPreProcessedCommandList(context);

		// Execute the commands in this list until one returns true
		// or throws an exception
		boolean saveResult = false;
		Exception saveException = null;
		int i = 0;
		int n = commandList.size();
		for (i = 0; i < n; i++) {
			try {
				saveResult = commandList.get(i).execute(context);
				if (saveResult) {
					break;
				}
			} catch (Exception e) {
				saveException = e;
				break;
			}
		}

		// Call postprocess methods on Filters in reverse order
		if (i >= n) { // Fell off the end of the chain
			i--;
		}
		boolean handled = false;
		boolean result = false;
		for (int j = i; j >= 0; j--) {
			Command command = commandList.get(j);
			if (command instanceof Filter) {
				try {
					result = ((Filter)command).postprocess(context, saveException);
					if (result) {
						handled = true;
					}
				} catch (Exception e) {
					// Silently ignore
				}
			}
		}

		// Return the exception or result state from the last execute()
		if ((saveException != null) && !handled) {
			throw saveException;
		} else {
			return (saveResult);
		}
	}

	/**
	 * Return the command list modified by all configured {@link DynamicChainProcessor}.
	 * 
	 * @return modified commandList.
	 */
	public List<Command> getPreProcessedCommandList(Context context) {
		List<Command> commandList = getRawCommandList();

		List<DynamicChainProcessor> processors =
				FacesChainFactory.getInstance().getDynamicChainProcessors();
		for (DynamicChainProcessor processor : processors) {
			commandList = processor.preProcess(commandList, context);
		}

		// preProcess the command chain
		return commandList;
	}

	/**
	 * Return the unchanged command list as configured for this chain.
	 * 
	 * @return the unchanged command list as configured for this chain.
	 */
	public List<Command> getRawCommandList() {
		return new CommandList(super.commands);
	}

	/**
	 * 
	 * @author A459429
	 * 
	 */
	static class CommandList extends ArrayList<Command> {

		private static final long serialVersionUID = 1L;

		public CommandList(Command[] commands) {
			super(commands.length);
			for (Command c : commands) {
				add(c);
			}
		}

		@Override
		public int indexOf(Object elem) {
			if (elem instanceof String) {
				String name = (String)elem;
				int m = size();
				for (int i = 0; i < m; i++) {
					Object o = get(i);
					if (o instanceof NamedCommand && name.equals(((NamedCommand)o).getName())) {
						return i;
					}
				}
				return -1;
			}
			return super.indexOf(elem);
		}

		@Override
		public boolean remove(Object o) {
			if (o instanceof String) {
				String name = (String)o;
				int m = size();
				for (int i = 0; i < m; i++) {
					Object elem = get(i);
					if (elem instanceof NamedCommand && name.equals(((NamedCommand)elem).getName())) {
						remove(i);
						return true;
					}
				}
				return false;
			}
			return super.remove(o);
		}

	}
}
