/**
 * Copyright (C) 2010 <dueni(at)gmx.net>
 * 
 * Licensed under the Apache License, Version 2.0 (the "License"); you may not use this file except
 * in compliance with the License. You may obtain a copy of the License at
 * 
 * http://www.apache.org/licenses/LICENSE-2.0
 * 
 * Unless required by applicable law or agreed to in writing, software distributed under the License
 * is distributed on an "AS IS" BASIS, WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express
 * or implied. See the License for the specific language governing permissions and limitations under
 * the License.
 */

package net.java.dev.faces.chain.handler;

import java.util.HashSet;
import java.util.Iterator;
import java.util.Set;

import javax.el.ELContext;
import javax.faces.bean.ManagedBean;
import javax.faces.bean.RequestScoped;
import javax.faces.context.FacesContext;

import org.apache.commons.chain.Catalog;
import org.apache.commons.chain.CatalogFactory;

/**
 * <code>ChainHandler</code> implements a request scoped bean to handle easy invocation of
 * commons-chain based chain of commands. The task of this bean is to handle {@link CatalogFactory}
 * access in an expression language friendly way.
 * 
 * @author dueni
 * 
 */
@ManagedBean(name = "chain")
@RequestScoped
public class ChainHandler extends AbstractChainMap<String, Object> {

	/**
	 * Private reference to the application scoped CatalogFactory.
	 */
	private CatalogFactory factory;

	/**
	 * the current ELContext.
	 */
	private ELContext elContext;

	/**
	 * Default constructor of ChainHandler.
	 */
	public ChainHandler() {
		this(null);
	}

	/**
	 * Construct an instance of ChainHandler for the passed in ELContext. If null is passed as
	 * argument, ELContext will be acquired from the current FacesContext.
	 * 
	 * <p>
	 * A reference to the application scoped CatalogFactory from application map will also be set.
	 * </p>
	 * 
	 * @param elContext
	 *          current ELContext or null to get it from the current FacesContext.
	 */
	public ChainHandler(ELContext elContext) {
		if (elContext == null) {
			this.elContext = FacesContext.getCurrentInstance().getELContext();
		} else {
			this.elContext = elContext;
		}
		factory = CatalogFactory.getInstance();
	}

	/**
	 * Returns true if CatalogFactory contains a Catalog for the key.
	 * 
	 * @return true if CatalogFactory contains a Catalog for the key.
	 */
	public boolean containsKey(Object key) {
		boolean contains = false;
		if (key instanceof String) {
			String name = (String)key;
			contains = (factory.getCatalog(name) != null);
			if (!contains && factory.getCatalog() != null) {
				contains = (factory.getCatalog().getCommand(name) != null);
			}
		}
		return contains;
	}

	/**
	 * Look for a Catalog for the given key and return a CatalogHandler wrapping the found Catalog. If
	 * no Catalog exists for the given key, the default Catalog will be used to look for a Command
	 * with that key and a CommandHandler for that command will be returned.
	 * 
	 * @return a CatalogHandler for found Catalog, CommandHandler for found Command on default Catalog
	 *         or an {@link UnknownCatalog} to handle misspelled key.
	 */
	public Object get(Object key) {
		Object result = null;
		if (key instanceof String) {
			String name = (String)key;
			// lookup catalog with given name
			Catalog catalog = factory.getCatalog(name);
			if (catalog != null) {
				result = new CatalogHandler(elContext, name, catalog);
			} else {
				// try lookup of command in default Catalog
				catalog = factory.getCatalog();
				if (catalog != null) {
					CatalogHandler catalogHandler = new CatalogHandler(elContext, "(default)", catalog);
					result = catalogHandler.get(name);
				} else {
					throw new IllegalArgumentException("Catalog for name '" + name + "' does not exist!");
				}
			}
		}
		return result;
	}

	/**
	 * Returns true if CatalogFactory has no named Catalogs loaded, false otherwise.
	 * 
	 * @return true if CatalogFactory has no named Catalogs loaded, false otherwise.
	 */
	public boolean isEmpty() {
		return !factory.getNames().hasNext();
	}

	/**
	 * Return a set of Catalog names.
	 * 
	 * @return a set of Catalog names.
	 */
	public Set<String> keySet() {
		Set<String> keySet = new HashSet<String>();
		Iterator<?> names = factory.getNames();
		while (names.hasNext()) {
			keySet.add((String)names.next());
		}
		return keySet;
	}

	/**
	 * Return the number of named Catalogs present in CatalogFactory.
	 * 
	 * @return the number of named Catalogs present in CatalogFactory.
	 */
	public int size() {
		int cnt = 0;
		Iterator<?> names = factory.getNames();
		while (names.hasNext()) {
			cnt++;
			names.next();
		}
		return cnt;
	}

}
